from __future__ import absolute_import, unicode_literals

from django.conf.urls import url
from wagtail_global_settings.views import global_settings, site_settings 


urlpatterns = [
    url(r'^global/$', global_settings.choose, name='wagtail_global_settings_choose'),
    url(r'^global/(?P<pk>\d+)/$', global_settings.edit, name='wagtail_global_settings_edit'),
    url(r'^site/$', site_settings.choose_site, name='wagtail_site_settings_choose_site'),
    url(r'^site/(?P<pk>\d+)/$', site_settings.choose, name='wagtail_site_settings_choose'),
    url(r'^site/(?P<pk>\d+)/(?P<ct_pk>\d+)$', site_settings.edit, name='wagtail_site_settings_edit'),
]



from __future__ import absolute_import, unicode_literals

from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes.models import ContentType
from solo.models import SingletonModel
from django.db import models

def _get_content_type(cls):
    if cls._content_type is None:
        cls._content_type = ContentType.objects.get_for_model(cls)
    return cls._content_type

def _get_global_settings_models(cls):
    if cls._global_settings_models is None:
        cls._global_settings_models = [model for model in cls.__subclasses__()]
    return cls._global_settings_models

def _get_global_settings_content_types(cls):
    if cls._global_settings_content_types is None:
        cls._global_settings_content_types = [model.get_content_type() for model in cls.get_global_settings_models()]
    return cls._global_settings_content_types

class AbstractGlobalSettingsCollection(SingletonModel):
    _global_settings_models = None
    _global_settings_content_types = None
    _content_type = None
    
    get_content_type = classmethod(_get_content_type)
    get_global_settings_models = classmethod(_get_global_settings_models)
    get_global_settings_content_types = classmethod(_get_global_settings_content_types)
    
    class Meta:
        abstract = True

class AbstractSiteSettingsCollectionQuerySet(models.QuerySet):
    def for_site(self, site):
        obj, created = self.get_or_create(site=site)  # @UnusedVariable
        return obj

class AbstractSiteSettingsCollection(models.Model):
    site = models.OneToOneField('wagtailcore.Site',
            null=False,
            blank=False,
            on_delete=models.CASCADE,
            related_name='+',
            verbose_name=_('Site'))
    
    objects = AbstractSiteSettingsCollectionQuerySet.as_manager()
    
    _global_settings_models = None
    _global_settings_content_types = None
    _content_type = None
    
    get_content_type = classmethod(_get_content_type)
    get_global_settings_models = classmethod(_get_global_settings_models)
    get_global_settings_content_types = classmethod(_get_global_settings_content_types)
    
    class Meta:
        abstract = True



from __future__ import unicode_literals, absolute_import

from django.conf.urls import include, url
from django.core import urlresolvers
from django.utils.translation import ugettext_lazy as _

from wagtail.wagtailcore import hooks
from wagtail.wagtailadmin.menu import MenuItem, SubmenuMenuItem

from wagtail_global_settings import urls
from wagtail_global_settings.permissions import user_can_edit_global_settings

class GlobalSettingsMenuItem(MenuItem):
    pass

class SiteSettingsMenuItem(MenuItem):
    pass

@hooks.register('register_admin_urls')
def register_admin_urls():
    return [
        url(r'^global-settings/', include(urls)),
    ]

@hooks.register('construct_main_menu')
def construct_main_menu(request, menu_items):
    if user_can_edit_global_settings(request.user):
        settings_menu_items = [item for item in menu_items if item.name == 'settings' and isinstance(item, SubmenuMenuItem)]
        if len(settings_menu_items) == 0:
            # settings menu not found
            g_menu_items = menu_items
        else:
            g_menu_items = settings_menu_items[0].menu.registered_menu_items
        
        global_settings_menu_items = [item for item in g_menu_items if item.name == 'global-settings' and isinstance(item, GlobalSettingsMenuItem)]
        if len(global_settings_menu_items) == 0:
            g_menu_items.append(
                GlobalSettingsMenuItem(_('Global settings'), urlresolvers.reverse('wagtail_global_settings_choose'),
                         classnames='icon icon-cog', name='global-settings', order=12000)
            )
        
        site_settings_menu_items = [item for item in g_menu_items if item.name == 'site-settings' and isinstance(item, SiteSettingsMenuItem)]
        if len(site_settings_menu_items) == 0:
            g_menu_items.append(
                SiteSettingsMenuItem(_('Site settings'), urlresolvers.reverse('wagtail_site_settings_choose_site'),
                         classnames='icon icon-cog', name='site-settings', order=10000)
            )



from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import permission_required
from django.shortcuts import redirect, render, get_object_or_404
from wagtail_global_settings.models import AbstractSiteSettingsCollection
from wagtail.wagtailcore.models import Site
from django.contrib.contenttypes.models import ContentType
from django.http.response import Http404
from django.utils.lru_cache import lru_cache
from wagtail.wagtailadmin.edit_handlers import extract_panel_definitions_from_model_class,\
    ObjectList, get_form_for_model
from django.contrib import messages

@lru_cache({}, 1)
def get_site_settings_edit_handler(model):
    panels = extract_panel_definitions_from_model_class(model, exclude=['site'])
    EditHandler = ObjectList(panels).bind_to_model(model)
    return EditHandler


@lru_cache({}, 2)
def get_site_settings_form(model, EditHandler):
    return get_form_for_model(
        model,
        formsets=EditHandler.required_formsets(),
        widgets=EditHandler.widget_overrides(),
        exclude=['site'])

@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def choose_site(request):
    site_settings_list = AbstractSiteSettingsCollection.get_global_settings_models()
    if len(site_settings_list) == 0:
        return render(request, 'wagtail_global_settings/choose_site.html', {
            'has_global_settings': False,
            'global_settings_list': []
        })
    sites = Site.objects.all()
    
    if sites.count() == 1:
        return redirect('wagtail_site_settings_choose', pk=sites.first().pk)

    return render(request, 'wagtail_global_settings/choose_site.html', {
        'has_site_settings': sites.count() != 0,
        'sites_list': sites,
    })

@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def choose(request, pk):
    site = get_object_or_404(Site, pk=pk)
    site_settings_list = AbstractSiteSettingsCollection.get_global_settings_models()
    if len(site_settings_list) == 1:
        return redirect('wagtail_site_settings_edit', pk=site.pk, ct_pk=site_settings_list[0].get_content_type().pk)

    return render(request, 'wagtail_global_settings/choose_site_setting.html', {
        'site': site,
        'has_global_settings': len(site_settings_list) != 0,
        'site_settings_list': ((global_settings.get_content_type(), global_settings._meta.verbose_name)
                           for global_settings in site_settings_list)
    })

@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def edit(request, pk, ct_pk):
    site = get_object_or_404(Site, pk=pk)
    content_type = get_object_or_404(ContentType, pk=ct_pk)
    model = content_type.model_class()
    if not issubclass(model, AbstractSiteSettingsCollection):
        raise Http404
    instance = model.objects.for_site(site)
    
    EditHandler = get_site_settings_edit_handler(model)
    EditForm = get_site_settings_form(model, EditHandler)

    if request.method == 'POST':
        form = EditForm(request.POST, request.FILES, instance=instance)

        if form.is_valid():
            instance = form.save(commit=False)
            instance.save()

            messages.success(request, _('The site settings "{0!s}" has been updated').format(instance._meta.verbose_name))
            return redirect('wagtail_site_settings_choose', pk=site.pk)
        else:
            messages.error(request, _('The global settings could not be updated due to validation errors'))
            edit_handler = EditHandler(instance=instance, form=form)
    else:
        form = EditForm(instance=instance)
        edit_handler = EditHandler(instance=instance, form=form)

    return render(request, 'wagtail_global_settings/edit_site_setting.html', {
        'site': site,
        'model_name': instance._meta.verbose_name,
        'content_type': content_type,
        'edit_handler': edit_handler,
    })



from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.decorators import permission_required
from django.shortcuts import redirect, render, get_object_or_404

from django.utils.lru_cache import lru_cache

from wagtail_global_settings.models import AbstractGlobalSettingsCollection
from django.contrib.contenttypes.models import ContentType
from wagtail.wagtailadmin.edit_handlers import extract_panel_definitions_from_model_class,\
    ObjectList, get_form_for_model
from django.contrib import messages
from django.http.response import Http404

@lru_cache({}, 1)
def get_global_settings_edit_handler(model):
    panels = extract_panel_definitions_from_model_class(model, exclude=[])
    EditHandler = ObjectList(panels).bind_to_model(model)
    return EditHandler


@lru_cache({}, 2)
def get_global_settings_form(model, EditHandler):
    return get_form_for_model(
        model,
        formsets=EditHandler.required_formsets(),
        widgets=EditHandler.widget_overrides(),
        exclude=[])

@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def choose(request):
    global_settings_list = AbstractGlobalSettingsCollection.get_global_settings_models()
    if len(global_settings_list) == 1:
        return redirect('wagtail_global_settings_edit', pk=global_settings_list[0].get_content_type().pk)

    return render(request, 'wagtail_global_settings/choose.html', {
        'has_global_settings': len(global_settings_list) != 0,
        'global_settings_list': ((global_settings.get_content_type(), global_settings._meta.verbose_name)
                           for global_settings in global_settings_list)
    })


@permission_required('wagtailadmin.access_admin')  # further permissions are enforced within the view
def edit(request, pk):
    content_type = get_object_or_404(ContentType, pk=pk)
    model = content_type.model_class()
    if not issubclass(model, AbstractGlobalSettingsCollection):
        raise Http404
    instance = model.get_solo()
    
    EditHandler = get_global_settings_edit_handler(model)
    EditForm = get_global_settings_form(model, EditHandler)

    if request.method == 'POST':
        form = EditForm(request.POST, request.FILES, instance=instance)

        if form.is_valid():
            instance = form.save(commit=False)
            instance.save()

            messages.success(request, _('The global settings "{0!s}" has been updated').format(instance._meta.verbose_name))
            return redirect('wagtail_global_settings_edit', pk=content_type.pk)
        else:
            messages.error(request, _('The global settings could not be updated due to validation errors'))
            edit_handler = EditHandler(instance=instance, form=form)
    else:
        form = EditForm(instance=instance)
        edit_handler = EditHandler(instance=instance, form=form)

    return render(request, 'wagtail_global_settings/edit.html', {
        'model_name': instance._meta.verbose_name,
        'content_type': content_type,
        'edit_handler': edit_handler,
    })



from django import template
from wagtail_global_settings.models import AbstractGlobalSettingsCollection

register = template.Library()

_global_settings_cache = None

def _get_components(app_label, model_name=None, field_name=None):
    if app_label is not None and model_name is None and field_name is None:
        components = unicode(app_label).split('.')
        if len(components) < 2:
            return u''
        app_label = components[0]
        model_name = components[1]
        field_name = components[2] if len(components) > 2 else None
    return app_label, model_name, field_name

def _get_global_settings_cache():
    global _global_settings_cache
    if _global_settings_cache is None:
        global_settings = {}
        for model in AbstractGlobalSettingsCollection.get_global_settings_models():
            if model.get_content_type().app_label not in global_settings:
                global_settings[model.get_content_type().app_label] = {}
            global_settings[model.get_content_type().app_label][model.__name__] = model
        _global_settings_cache = global_settings
    return _global_settings_cache

def _get_global_setting(app_label, model_name, field_name=None):
    _global_settings = _get_global_settings_cache()
    if app_label in _global_settings and model_name in _global_settings[app_label]:
        model = _global_settings[app_label][model_name]
        if issubclass(model, AbstractGlobalSettingsCollection):
            instance = model.get_solo()
            if field_name is None:
                return instance
            else:
                return getattr(instance, field_name, u'')
        else:
            return u''
    else:
        return u''

@register.simple_tag(name='global_settings')
def _global_settings(app_label, model_name=None, field_name=None):
    return _get_global_setting(*_get_components(app_label, model_name, field_name))

@register.assignment_tag(name='get_global_settings')
def _get_global_settings(app_label, model_name=None, field_name=None):
    return _get_global_setting(*_get_components(app_label, model_name, field_name))



from wagtail_global_settings.models import AbstractGlobalSettingsCollection,\
    AbstractSiteSettingsCollection

def global_settings(request):
    settings = {}
    for model in AbstractGlobalSettingsCollection.get_global_settings_models():
        if model.get_content_type().app_label not in settings:
            settings[model.get_content_type().app_label] = {}
        settings[model.get_content_type().app_label][model.__name__] = model.get_solo()
    return {
        'global_settings': settings
    }

def site_settings(request):
    if request is None or request.site is None:
        raise Exception, "Please add 'wagtail_global_settings.context_processors.site_settings' after 'django.template.context_processors.request'"
    settings = {}
    for model in AbstractSiteSettingsCollection.get_global_settings_models():
        if model.get_content_type().app_label not in settings:
            settings[model.get_content_type().app_label] = {}
        settings[model.get_content_type().app_label][model.__name__] = model.objects.for_site(request.site)
    return {
        'site_settings': settings
    }



from django import template
from wagtail_global_settings.models import AbstractSiteSettingsCollection
from wagtail.wagtailcore.models import Site

register = template.Library()

_site_settings_cache = None

def _get_components(app_label, model_name=None, field_name=None):
    if app_label is not None and model_name is None and field_name is None:
        components = unicode(app_label).split('.')
        if len(components) < 2:
            return u''
        app_label = components[0]
        model_name = components[1]
        field_name = components[2] if len(components) > 2 else None
    return app_label, model_name, field_name

def _get_site_settings_cache():
    global _site_settings_cache
    if _site_settings_cache is None:
        site_settings = {}
        for model in AbstractSiteSettingsCollection.get_global_settings_models():
            if model.get_content_type().app_label not in site_settings:
                site_settings[model.get_content_type().app_label] = {}
            site_settings[model.get_content_type().app_label][model.__name__] = model
        _site_settings_cache = site_settings
    return _site_settings_cache

def _get_site_setting(site, app_label, model_name, field_name=None):
    site_settings = _get_site_settings_cache()
    if app_label in site_settings and model_name in site_settings[app_label]:
        model = site_settings[app_label][model_name]
        if issubclass(model, AbstractSiteSettingsCollection):
            instance = model.objects.for_site(site)
            if field_name is None:
                return instance
            else:
                return getattr(instance, field_name, u'')
        else:
            return u''
    else:
        return u''

@register.simple_tag(name='site_settings', takes_context=True)
def _site_settings(context, app_label, model_name=None, field_name=None, site=None):
    if site is None or not isinstance(site, Site):
        site = context['request'].site
    return _get_site_setting(site, *_get_components(app_label, model_name, field_name))

@register.assignment_tag(name='get_site_settings', takes_context=True)
def _get_site_settings(context, app_label, model_name=None, field_name=None, site=None):
    if site is None or not isinstance(site, Site):
        site = context['request'].site
    return _get_site_setting(site, *_get_components(app_label, model_name, field_name))



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

if os.path.exists('README.txt'):
    README = open('README.txt').read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='wagtail-global-settings',
    version='0.1.2',
    packages=find_packages(exclude=[]),
    package_data={'wagtail_global_settings': ['templates/wagtail_global_settings/*.html']},
    install_requires=['wagtail', 'django-solo'],
    include_package_data=True,
    license='BSD License',
    description='Global settings editor for Wagtail using django-solo',
    long_description=README,
    url='https://bitbucket.org/jordanmarkov/wagtail-global-settings',
    download_url='https://bitbucket.org/jordanmarkov/wagtail-global-settings/get/0.1.2.tar.gz',
    author='Jordan Markov',
    author_email='jmarkov@gmail.com',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)
